The code in this folder generates the data for the plots seen in the *Ergodic Bounds* section of "A Decentralized Receiver in Gaussian Interference." 

# Code Structure 
## Data collection code 
 - `run_make_graphs.py`: Sample some observations which, when averaged, will produce the plots from the paper. Put these observations into a dictionary and write it to a file in `samples/`.
 - `collect_data_forever.sh`: Run `run_make_graphs.py` over and over again to collect many observations.
 - `average_samples.py`: Accumulate all the observations in `samples/` and average them, producing the plot points we see in the paper. Write a CSV of each plot to an appropriately named file in `plot csv/`. Each CSV has been named according to the ergodic bound plot it contains. The "CSV Filename Key" section explains our naming convention.
 - `accum_samples.py`: Accumulate all the observations in all the samples in `samples/` into a single giant CSV in `plot csv/` which can be easily imported into, say, R, for data analysis. See "`accum_samples.py` CSV Column Key" section.
 - `make_graphs/`: Sample the paper's bounds for a Ricean channel with some parameters.
	- `plot.py`: Generate a series of rates versus K-factor or LAN constraint, return a dictionary. Methods `vs_bits(...)` and `vs_k(...)` can be run standalone to produce non-averaged plots if some blocks are un-commented.
	- `sampling.py`: Generate rate-vs-bits data points over trials of a random 0dB-average-SNR channel using `plot.py`.

## Bound computation code
 - `dist_recv/`: Cython code that performs maximization on the distributed receive bounds for a particular channel.
	- `rates.pyx`: Objective functions to optimize over
	- `opt_rates.pyx`: Perform optimization of the objective functions from `rates.pyx`
	- `checkdc.pyx`: Check whether a distributed compression configuration violates entropy conditions. In other words, for a given channel, rate vector $\vec{r}$ and pre-compression-rate vector $\vec{\rho}$, determine whether or not $\vec{\rho} \in R_{DC}(\vec{r})$.
 - `test_dist_recv`: Some sanity checks for functions inside `dist_recv/`

# CSV Filename Key
The CSVs in `plot csv/` are named according to the scheme:

    [bits|k]_[i|ni]_[b|nb]_[ub|bc|ga|dc].csv

* `[bits|k]`: The variable being plotted against. `bits` means plot varies the cumulative amount of bits the receivers forward to the base, `k` means the channel's K-factor varies.
* `[i|ni]`: 'interferer' or 'no interferer', indicates the presence of a 20dB interferer.
* `[b|nb]`: 'base' or 'no base',  indicates whether or not the base gets its own observations of the transmission in addition to information provided by helpers.
* `[ub|bc|ga|dc]`: `upper bound`, `broadcast`, `gaussian approximation` or `distributed compression`, indicates the bound being plotted.

# `accum_samples.py` CSV Column Key 
The CSVs produced by `accum_samples.py` have one observation of each of the bounds per row. The columns are the parameters used to generate each observation as well as each bound.

* `k`: K-factor the channel was generated with
* `inr`: Interference-to-noise ratio (dB) the channel was generated with.
* `snr`: Signal-to-noise ratio (dB) the channel was generated with.
* `bits`: LAN constraint (bits) the bounds were computed with.
* `ub`: Upper bound on communications rate
* `bc`: Broadcast channel achievable rate
* `ga`: Gaussian approximation channel achievable rate
* `dc`: Distributed compression achievable rate
* `v_dc_sorted`: Allocation of LAN bits to each helper, sorted from high to low
* `base`: Boolean, whether or not the base gets its own observations of the transmission in addition to information provided by helpers.
