""""Average samples produced by run_make_graphs.py"""
import numpy as np
from datetime import datetime
from os import listdir
import sys
from scipy import spatial

from make_graphs import plot
from make_graphs import sampling


def function_hull(v_x, v_y):
    """Linear hull of a non-negative function"""
    d_xmax = max(v_x)
    nda_graph = np.array([np.append(v_x, [0, d_xmax]),
                          np.append(v_y, [0, 0])]).T
    hull = spatial.ConvexHull(nda_graph)
    v_hull_idxs = np.sort(hull.vertices)

    if (v_hull_idxs[-1] == len(v_x)) | (v_hull_idxs[-1] == len(v_x)+1):
        v_hull_idxs = v_hull_idxs[:-1]
    if (v_hull_idxs[-1] == len(v_x)) | (v_hull_idxs[-1] == len(v_x)+1):
        v_hull_idxs = v_hull_idxs[:-1]

    v_hull = np.interp(v_x,
                       nda_graph[v_hull_idxs, 0],
                       nda_graph[v_hull_idxs, 1])
    return v_hull


def _average_samples_bits(str_prefix):
    dict_avg = dict()
    v_files = listdir('samples/')
    for str_fname in v_files:
        if str_fname.startswith(str_prefix):
            sam = np.load('samples/'+str_fname).tolist()

            d_kfactor = sam['input']['v_kfactors'][0]
            d_inr = sam['input']['v_inrs'][0]
            v_max_bits = sam['input']['v_max_bits']
            n_trials = sam['input']['n_trials']

            # Update loaded sample (sam) to convex hulls of each series of bits
            n_pts = len(v_max_bits)
            v_cfgs = [(d_kfactor, d_inr, v_max_bits[n_pt]) for n_pt in range(n_pts)]
            for n_trial in range(n_trials):
                v_trial_dc_rates = [sam[t_cfg][n_trial].rate_opt_dc for t_cfg in v_cfgs]
                v_trial_ga_rates = [sam[t_cfg][n_trial].rate_opt_ga for t_cfg in v_cfgs]
                v_dc_hull_rates = function_hull(v_max_bits, v_trial_dc_rates)
                v_ga_hull_rates = function_hull(v_max_bits, v_trial_ga_rates)
                for iii in range(0, len(v_max_bits)):
                    sam[v_cfgs[iii]][n_trial].rate_opt_dc = v_dc_hull_rates[iii]
                    sam[v_cfgs[iii]][n_trial].rate_opt_ga = v_ga_hull_rates[iii]

            dict_avg = sampling.sample_average(sam, dict_avg)

    dict_plot = plot.vs_bits(dict_avg=dict_avg)
    print(dict_plot['n_trials'])
    return dict_plot

def _average_samples_k(str_prefix):
    dict_avg = dict()
    n_total_trials = 0
    v_files = listdir('samples/')
    for str_fname in v_files:
        if str_fname.startswith(str_prefix):
            sam = np.load('samples/'+str_fname).tolist()
            dict_avg = sampling.sample_average(sam, dict_avg)

    dict_plot = plot.vs_k(dict_avg=dict_avg)
    print(dict_plot['n_trials'])
    return dict_plot


if __name__ == "__main__":
    startTime = datetime.now()

    n_recv = 5

    str_prefix = 'bits_ni_nb_'
    print(str_prefix)
    dict_bits_ni_nb = _average_samples_bits(str_prefix)
    v_bits = dict_bits_ni_nb['x_axis']
    with open('plot csv/bits_ni_nb_ub.csv', 'wb') as f:
        np.savetxt(f, np.array([v_bits, dict_bits_ni_nb['ub']]).T, delimiter=',')
    with open('plot csv/bits_ni_nb_bc.csv', 'wb') as f:
        np.savetxt(f, np.array([v_bits, dict_bits_ni_nb['bc']]).T, delimiter=',')
    with open('plot csv/bits_ni_nb_ga.csv', 'wb') as f:
        np.savetxt(f, np.array([v_bits, dict_bits_ni_nb['ga']]).T, delimiter=',')
    with open('plot csv/bits_ni_nb_dc.csv', 'wb') as f:
        np.savetxt(f, np.array([v_bits, dict_bits_ni_nb['dc']]).T, delimiter=',')

    str_prefix = 'bits_ni_b_'
    print(str_prefix)
    dict_bits_ni_b = _average_samples_bits(str_prefix)
    v_bits = dict_bits_ni_b['x_axis']
    with open('plot csv/bits_ni_b_ub.csv', 'wb') as f:
        np.savetxt(f, np.array([v_bits, dict_bits_ni_b['ub']]).T, delimiter=',')
    with open('plot csv/bits_ni_b_bc.csv', 'wb') as f:
        np.savetxt(f, np.array([v_bits, dict_bits_ni_b['bc']]).T, delimiter=',')
    with open('plot csv/bits_ni_b_ga.csv', 'wb') as f:
        np.savetxt(f, np.array([v_bits, dict_bits_ni_b['ga']]).T, delimiter=',')
    with open('plot csv/bits_ni_b_dc.csv', 'wb') as f:
        np.savetxt(f, np.array([v_bits, dict_bits_ni_b['dc']]).T, delimiter=',')

    str_prefix = 'bits_i_nb_'
    print(str_prefix)
    dict_bits_i_nb = _average_samples_bits(str_prefix)
    v_bits = dict_bits_i_nb['x_axis']
    with open('plot csv/bits_i_nb_ub.csv', 'wb') as f:
        np.savetxt(f, np.array([v_bits, dict_bits_i_nb['ub']]).T, delimiter=',')
    with open('plot csv/bits_i_nb_bc.csv', 'wb') as f:
        np.savetxt(f, np.array([v_bits, dict_bits_i_nb['bc']]).T, delimiter=',')
    with open('plot csv/bits_i_nb_ga.csv', 'wb') as f:
        np.savetxt(f, np.array([v_bits, dict_bits_i_nb['ga']]).T, delimiter=',')
    with open('plot csv/bits_i_nb_dc.csv', 'wb') as f:
        np.savetxt(f, np.array([v_bits, dict_bits_i_nb['dc']]).T, delimiter=',')

    str_prefix = 'bits_i_b_'
    print(str_prefix)
    dict_bits_i_b = _average_samples_bits(str_prefix)
    v_bits = dict_bits_i_b['x_axis']
    with open('plot csv/bits_i_b_ub.csv', 'wb') as f:
        np.savetxt(f, np.array([v_bits, dict_bits_i_b['ub']]).T, delimiter=',')
    with open('plot csv/bits_i_b_bc.csv', 'wb') as f:
        np.savetxt(f, np.array([v_bits, dict_bits_i_b['bc']]).T, delimiter=',')
    with open('plot csv/bits_i_b_ga.csv', 'wb') as f:
        np.savetxt(f, np.array([v_bits, dict_bits_i_b['ga']]).T, delimiter=',')
    with open('plot csv/bits_i_b_dc.csv', 'wb') as f:
        np.savetxt(f, np.array([v_bits, dict_bits_i_b['dc']]).T, delimiter=',')

    str_prefix = 'k_ni_nb_'
    print(str_prefix)
    dict_k_ni_nb = _average_samples_k(str_prefix)
    v_kfactor = dict_k_ni_nb['x_axis']
    with open('plot csv/k_ni_nb_ub.csv', 'wb') as f:
        np.savetxt(f, np.array([v_kfactor, dict_k_ni_nb['ub']]).T, delimiter=',')
    with open('plot csv/k_ni_nb_bc.csv', 'wb') as f:
        np.savetxt(f, np.array([v_kfactor, dict_k_ni_nb['bc']]).T, delimiter=',')
    with open('plot csv/k_ni_nb_ga.csv', 'wb') as f:
        np.savetxt(f, np.array([v_kfactor, dict_k_ni_nb['ga']]).T, delimiter=',')
    with open('plot csv/k_ni_nb_dc.csv', 'wb') as f:
        np.savetxt(f, np.array([v_kfactor, dict_k_ni_nb['dc']]).T, delimiter=',')
    nda_bitdistns = np.append([v_kfactor], [dict_k_ni_nb[iii] for iii in range(n_recv - 1)], 0)
    with open('plot csv/k_ni_nb_dist.csv', 'wb') as f:
        np.savetxt(f, nda_bitdistns.T, delimiter=',')

    str_prefix = 'k_ni_b_'
    print(str_prefix)
    dict_k_ni_b = _average_samples_k(str_prefix)
    v_kfactor = dict_k_ni_b['x_axis']
    with open('plot csv/k_ni_b_ub.csv', 'wb') as f:
        np.savetxt(f, np.array([v_kfactor, dict_k_ni_b['ub']]).T, delimiter=',')
    with open('plot csv/k_ni_b_bc.csv', 'wb') as f:
        np.savetxt(f, np.array([v_kfactor, dict_k_ni_b['bc']]).T, delimiter=',')
    with open('plot csv/k_ni_b_ga.csv', 'wb') as f:
        np.savetxt(f, np.array([v_kfactor, dict_k_ni_b['ga']]).T, delimiter=',')
    with open('plot csv/k_ni_b_dc.csv', 'wb') as f:
        np.savetxt(f, np.array([v_kfactor, dict_k_ni_b['dc']]).T, delimiter=',')
    nda_bitdistns = np.append([v_kfactor], [dict_k_ni_b[iii] for iii in range(n_recv - 1)], 0)
    with open('plot csv/k_ni_b_dist.csv', 'wb') as f:
        np.savetxt(f, nda_bitdistns.T, delimiter=',')

    str_prefix = 'k_i_nb_'
    print(str_prefix)
    dict_k_i_nb = _average_samples_k(str_prefix)
    v_kfactor = dict_k_i_nb['x_axis']
    with open('plot csv/k_i_nb_ub.csv', 'wb') as f:
        np.savetxt(f, np.array([v_kfactor, dict_k_i_nb['ub']]).T, delimiter=',')
    with open('plot csv/k_i_nb_bc.csv', 'wb') as f:
        np.savetxt(f, np.array([v_kfactor, dict_k_i_nb['bc']]).T, delimiter=',')
    with open('plot csv/k_i_nb_ga.csv', 'wb') as f:
        np.savetxt(f, np.array([v_kfactor, dict_k_i_nb['ga']]).T, delimiter=',')
    with open('plot csv/k_i_nb_dc.csv', 'wb') as f:
        np.savetxt(f, np.array([v_kfactor, dict_k_i_nb['dc']]).T, delimiter=',')
    nda_bitdistns = np.append([v_kfactor], [dict_k_i_nb[iii] for iii in range(n_recv - 1)], 0)
    with open('plot csv/k_i_nb_dist.csv', 'wb') as f:
        np.savetxt(f, nda_bitdistns.T, delimiter=',')

    str_prefix = 'k_i_b_'
    print(str_prefix)
    dict_k_i_b = _average_samples_k(str_prefix)
    v_kfactor = dict_k_i_b['x_axis']
    with open('plot csv/k_i_b_ub.csv', 'wb') as f:
        np.savetxt(f, np.array([v_kfactor, dict_k_i_b['ub']]).T, delimiter=',')
    with open('plot csv/k_i_b_bc.csv', 'wb') as f:
        np.savetxt(f, np.array([v_kfactor, dict_k_i_b['bc']]).T, delimiter=',')
    with open('plot csv/k_i_b_ga.csv', 'wb') as f:
        np.savetxt(f, np.array([v_kfactor, dict_k_i_b['ga']]).T, delimiter=',')
    with open('plot csv/k_i_b_dc.csv', 'wb') as f:
        np.savetxt(f, np.array([v_kfactor, dict_k_i_b['dc']]).T, delimiter=',')
    nda_bitdistns = np.append([v_kfactor], [dict_k_i_b[iii] for iii in range(n_recv - 1)], 0)
    with open('plot csv/k_i_b_dist.csv', 'wb') as f:
        np.savetxt(f, nda_bitdistns.T, delimiter=',')

    print(datetime.now() - startTime)
    sys.exit(0)
