"""Make plots"""
import numpy as np
import itertools
import matplotlib.pyplot as plt

from make_graphs import sampling


def vs_bits(n_recv=5,
            d_kfactor=0,
            d_inr=0,
            v_max_bits=range(0, 10, 2),
            n_trials=1,
            b_no_base=False,
            n_detail_lvl=0,
            dict_avg=None,
            dict_sample=None):
    """ Given a sample average, make a plot vs lan bits
    Input:
        dict_avg = averaged vs-bits data to plot
        ONLY IF YOU ARE NOT PASSING IN A DICT_SAMPLE:
        n_recv = number of receivers
        d_kfactor = Channel K-factor (dB) to run trials on (unique elements)
        d_inr = INR (dB) to run trials on (unique elements)
        v_max_bits = sum-bitrates to run trials on (unique elements)
        n_trials = Number of trials to run for each combination of params in v_kfactors, v_inr, v_lanbits
    Output:
        dict_plot = graph vs bits
        dict_avg = dict with properly averaged plot information (i.e. with appropriate hulls taken during averaging)
    """
    dict_sample = None
    if dict_avg is None:
        dict_sample = sampling.random_sample(n_recv,
                                             [d_kfactor],
                                             [d_inr],
                                             v_max_bits,
                                             n_trials,
                                             b_no_base,
                                             n_detail_lvl)
        dict_avg = sampling.sample_average(dict_sample)
    else:
        v_max_bits = dict_avg['input']['v_max_bits']
        d_kfactor = dict_avg['input']['v_kfactors'][0]
        d_inr = dict_avg['input']['v_inrs'][0]

    n_pts = len(dict_avg['input']['v_max_bits'])
    v_avg_ub = np.zeros(n_pts)
    v_avg_bc = np.zeros(n_pts)
    v_avg_opt_ga = np.zeros(n_pts)
    v_avg_opt_dc = np.zeros(n_pts)

    for n_pt in range(n_pts):
        t_cfg = (d_kfactor, d_inr, v_max_bits[n_pt])
        v_avg_ub[n_pt] = dict_avg[t_cfg].rate_ub
        v_avg_bc[n_pt] = dict_avg[t_cfg].rate_bc
        v_avg_opt_ga[n_pt] = dict_avg[t_cfg].rate_opt_ga
        v_avg_opt_dc[n_pt] = dict_avg[t_cfg].rate_opt_dc

    """
    plt.gca().set_prop_cycle(color=['red', 'green', 'blue', 'yellow'])
    plt.plot(v_max_bits, v_avg_ub)
    plt.plot(v_max_bits, v_avg_bc)
    plt.plot(v_max_bits, v_avg_opt_ga)
    plt.plot(v_max_bits, v_avg_opt_dc)

    plt.legend(['Upper', 'Broadcast', 'Gaussian', 'DC'], loc='upper left')

    plt.show()
    """

    dict_plot = {'x_axis': v_max_bits,
                 'ub': v_avg_ub,
                 'bc': v_avg_bc,
                 'ga': v_avg_opt_ga,
                 'dc': v_avg_opt_dc,
                 'n_trials': dict_avg['n_trials']}
    if dict_avg is None:
        return dict_plot, dict_sample
    else:
        return dict_plot


def vs_k(n_recv=5,
         v_kfactor=[-40, -20, 0, 20, 40],
         d_inr=0,
         d_max_bits=5,
         n_trials=1,
         b_no_base=False,
         n_detail_lvl=0,
         dict_avg=None):
    """Given a sample average, make a plot vs lan bits
    Input:
        dict_avg = averaged vs-k data to plot
        ONLY IF AVERAGE DATA IS NOT SPECIFIED:
        n_recv = number of receivers
        d_kfactor = Channel K-factor (dB) to run trials on (unique elements)
        d_inr = INR (dB) to run trials on (unique elements)
        d_max_bits = sum-bitrates to run trials on  (unique elements)
        n_trials = Number of trials to run for each combination of params in v_kfactors, v_inr, v_lanbits
    Output:
        dict_plot = dict containing plot points
    """
    dict_sample = None
    if dict_avg is None:
        dict_sample = sampling.random_sample(n_recv,
                                             v_kfactor,
                                             [d_inr],
                                             [d_max_bits],
                                             n_trials,
                                             b_no_base,
                                             n_detail_lvl)
        dict_avg = sampling.sample_average(dict_sample)
    else:
        v_kfactor = dict_avg['input']['v_kfactors']
        d_inr = dict_avg['input']['v_inrs'][0]
        d_max_bits = dict_avg['input']['v_max_bits'][0]

    n_pts = len(dict_avg['input']['v_kfactors'])

    v_avg_ub = np.zeros(n_pts)
    v_avg_bc = np.zeros(n_pts)
    v_avg_opt_ga = np.zeros(n_pts)
    v_avg_opt_dc = np.zeros(n_pts)
    nda_dc_sorted = np.zeros((n_pts, n_recv-1))

    for n_pt in range(n_pts):
        t_cfg = (v_kfactor[n_pt], d_inr, d_max_bits)
        v_avg_ub[n_pt] = dict_avg[t_cfg].rate_ub
        v_avg_bc[n_pt] = dict_avg[t_cfg].rate_bc
        v_avg_opt_ga[n_pt] = dict_avg[t_cfg].rate_opt_ga
        v_avg_opt_dc[n_pt] = dict_avg[t_cfg].rate_opt_dc
        nda_dc_sorted[n_pt, :] = dict_avg[t_cfg].v_dc_sorted

    """"
    plt.gca().set_prop_cycle(color=['red', 'green', 'blue', 'yellow'])
    plt.plot(v_kfactor, v_avg_ub)
    plt.plot(v_kfactor, v_avg_bc)
    plt.plot(v_kfactor, v_avg_opt_ga)
    plt.plot(v_kfactor, v_avg_opt_dc)

    plt.legend(['Upper', 'Broadcast', 'Gaussian', 'DC'], loc='upper left')

    plt.show()
    """
    dict_plot = {'x_axis': v_kfactor,
                 'ub': v_avg_ub,
                 'bc': v_avg_bc,
                 'ga': v_avg_opt_ga,
                 'dc': v_avg_opt_dc,
                 'n_trials': dict_avg['n_trials']}

    for iii in range(n_recv-1):
        dict_plot[iii] = nda_dc_sorted[:, iii]

    if dict_avg is None:
        return dict_plot, dict_sample
    else:
        return dict_plot
