"""Tests for checkdc, a method that checks whether a distributed compression configuration is sane"""
import numpy as np

from dist_recv import checkdc as dc


def test_trivial_dc_config():
    """Make sure v_ga_bits == v_dc_bits config is valid"""
    n_recv = 5
    v_channel = 1j*np.random.standard_normal(n_recv) + np.random.standard_normal(n_recv)
    v_channel = v_channel*1/np.sqrt(2)
    nda_interf = 1j*np.random.standard_normal((n_recv, n_recv)) + \
                    np.random.standard_normal((n_recv, n_recv))
    nda_interf = np.dot(nda_interf, nda_interf.conj().T)/2
    v_bits = np.random.standard_normal(n_recv-1)**2
    d_lambda = 0
    assert dc.checkdc(v_channel, nda_interf, v_bits, v_bits, d_lambda)


def test_better_dc_config():
    """"Make sure there is an eps>0 where v_ga_bits = (1+eps)*v_dc_bits is valid"""
    n_recv = 5
    v_channel = 1j*np.random.standard_normal(n_recv) + np.random.standard_normal(n_recv)
    v_channel = v_channel*1/np.sqrt(2)
    nda_interf = 1j*np.random.standard_normal((n_recv, n_recv)) + \
                    np.random.standard_normal((n_recv, n_recv))
    nda_interf = np.dot(nda_interf, nda_interf.conj().T)/2
    v_bits = np.random.standard_normal(n_recv-1)**2 + 1
    d_lambda = 0
    d_small_number = 10e-4
    assert dc.checkdc(v_channel, nda_interf, v_bits, (1+d_small_number)*v_bits, d_lambda)
