import numpy as np

from dist_recv import rates
from dist_recv import opt_rates

def test_opt_rate_ga():
    """Make sure opt_rate_ga is better than 100 random configurations"""
    n_recv = 5
    n_helpers = n_recv-1
    v_channel = 1j*np.random.standard_normal(n_recv) + np.random.standard_normal(n_recv)
    v_channel = v_channel*1/np.sqrt(2)
    nda_interf = 1j*np.random.standard_normal((n_recv, n_recv)) + \
                 np.random.standard_normal((n_recv, n_recv))
    nda_interf = np.dot(nda_interf, nda_interf.conj().T)/2
    d_max_bits = 5

    opt_result = opt_rates.opt_ga(v_channel, nda_interf, d_max_bits)
    d_opt_rate = opt_rates.rate_opt_ga(v_channel, nda_interf, opt_result)

    d_best_random_rate = 0
    for iii in range(0, 100):
        v_random_bit_config = np.random.rand(n_helpers)
        d_sum = sum(v_random_bit_config)
        v_random_config = v_random_bit_config*d_max_bits/d_sum
        d_random_rate = rates.rate_ga(v_channel, nda_interf, v_random_config)
        if d_random_rate > d_best_random_rate:
            d_best_random_rate = d_random_rate

    assert d_opt_rate >= d_best_random_rate


def test_opt_rate_dc_geq_ga():
    """Make sure opt_rate_dc is better than opt_rate_ga"""
    n_recv = 5
    n_helpers = n_recv-1
    v_channel = 1j*np.random.standard_normal(n_recv) + np.random.standard_normal(n_recv)
    v_channel = v_channel*1/np.sqrt(2)
    nda_interf = 1j*np.random.standard_normal((n_recv, n_recv)) + \
                 np.random.standard_normal((n_recv, n_recv))
    nda_interf = np.dot(nda_interf, nda_interf.conj().T)/2
    d_max_bits = 5

    opt_result_ga = opt_rates.opt_ga(v_channel, nda_interf, d_max_bits)
    d_opt_ga_rate = opt_rates.rate_opt_ga(v_channel, nda_interf, opt_result_ga)

    opt_result_dc = opt_rates.opt_dc(v_channel, nda_interf, d_max_bits)
    d_opt_dc_rate = opt_rates.rate_opt_dc(v_channel, nda_interf, opt_result_dc)

    assert d_opt_dc_rate >= d_opt_ga_rate


def test_opt_rate_dc_leq_ub():
    """Make sure opt_rate_dc is less than upper bound"""
    n_recv = 5
    n_helpers = n_recv-1
    v_channel = 1j*np.random.standard_normal(n_recv) + np.random.standard_normal(n_recv)
    v_channel = v_channel*1/np.sqrt(2)
    nda_interf = 1j*np.random.standard_normal((n_recv, n_recv)) + \
                 np.random.standard_normal((n_recv, n_recv))
    nda_interf = np.dot(nda_interf, nda_interf.conj().T)/2
    d_max_bits = 5

    d_ub = rates.rate_ub(v_channel, nda_interf, d_max_bits)

    opt_result_dc = opt_rates.opt_dc(v_channel, nda_interf, d_max_bits)
    d_opt_dc_rate = opt_rates.rate_opt_dc(v_channel, nda_interf, opt_result_dc)

    assert d_ub >= d_opt_dc_rate


def test_opt_rate_ga_leq_ub():
    """Make sure opt_rate_ga is less than upper bound"""
    n_recv = 5
    n_helpers = n_recv-1
    v_channel = 1j*np.random.standard_normal(n_recv) + np.random.standard_normal(n_recv)
    v_channel = v_channel*1/np.sqrt(2)
    nda_interf = 1j*np.random.standard_normal((n_recv, n_recv)) + \
                 np.random.standard_normal((n_recv, n_recv))
    nda_interf = np.dot(nda_interf, nda_interf.conj().T)/2
    d_max_bits = 5

    d_ub = rates.rate_ub(v_channel, nda_interf, d_max_bits)

    opt_result_ga = opt_rates.opt_ga(v_channel, nda_interf, d_max_bits)
    d_opt_ga_rate = opt_rates.rate_opt_ga(v_channel, nda_interf, opt_result_ga)

    assert d_ub >= d_opt_ga_rate
